# -*- coding: utf-8 -*-
# <Couleur - fancy shell output for python>
# Copyright (C) <2010>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import uuid

__version__ = '0.1.2'

def ansify(number):
    """Wraps the given ansi code to a proper escaped python output

    Arguments:
    - `number`: the code in question
    """
    number = unicode(number)
    return '\033[%sm' % number

class modifiers:
    reset = ansify(0)
    bold = ansify(1)
    blink = ansify(2)
    italic = ansify(3)
    underline = ansify(4)
    inverse = ansify(7)
    strikethrough = ansify(9)
    up = '\r\033[A'

    class off:
        bold = ansify(22)
        italic = ansify(23)
        underline = ansify(24)
        inverse = ansify(27)
        strikethrough = ansify(29)

class forecolors:
    black = ansify(30)
    red = ansify(31)
    green = ansify(32)
    yellow = ansify(33)
    blue = ansify(34)
    magenta = ansify(35)
    cyan = ansify(36)
    white = ansify(37)
    normal = ansify(39)

class backcolors:
    black = ansify(40)
    red = ansify(41)
    green = ansify(42)
    yellow = ansify(43)
    blue = ansify(44)
    magenta = ansify(45)
    cyan = ansify(46)
    white = ansify(47)
    normal = ansify(49)

def fore(color):
    def get(what):
        try:
            r = getattr(modifiers, what)
        except AttributeError:
            r = getattr(forecolors, what)
        return r

    args = map(get, color.split("_"))
    return "".join(args)

def back(color):
    return getattr(backcolors, color)

_sep1 = '_on_'
_sep2 = '_and_'

class Shell(object):
    def __init__(self, indent=2, linebreak=False, bold=False):
        self._indentation_factor = indent
        self._indent = 0
        self._linebreak = linebreak
        self._bold = bold
        self._in_format = False

    def indent(self):
        self._indent += self._indentation_factor

    def dedent(self):
        self._indent -= self._indentation_factor

    def _printer_for(self, color):
        colors = color.split(_sep1)

        parts = [
            fore(colors.pop(0)),
            "%s",
            modifiers.reset
        ]

        if colors:
            parts.insert(0, back(colors[0]))

        if not self._in_format:
            if self._bold:
                parts.insert(0, ansify(1))

            if self._indent:
                parts.insert(0, ' ' * self._indent)

            if self._linebreak:
                parts.append("\n")

        string = "".join(parts)

        def dec(z, replace=False):
            pre = (replace and modifiers.up or '')
            sys.stdout.write(pre + (string % z))

        return dec


    def __getattr__(self, attr):
        if not attr.startswith("_"):
            if _sep2 in attr:
                self._in_format = True
                printers = map(self._printer_for, attr.split(_sep2))
                def dec(string, replace=False):
                    unique = str(uuid.uuid4())
                    string = string.replace(r'\|', unique)
                    parts = string.split("|")
                    if replace:
                        sys.stdout.write(modifiers.up)

                    if self._indent:
                        sys.stdout.write(' ' * self._indent)

                    if self._bold:
                        sys.stdout.write(ansify(1))

                    for part, output in zip(parts, printers):
                        output(part.replace(unique, "|"))

                    if self._linebreak:
                        sys.stdout.write("\n")

                    self._in_format = False

                return dec

            return self._printer_for(attr)

        return super(Shell, self).__getattribute__(attr)



# -*- coding: utf-8 -*-
# <Couleur - fancy shell output for python>
# Copyright (C) <2010>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from StringIO import StringIO
from nose.tools import with_setup, assert_equals

from couleur import ansify
from couleur import Shell

def prepare_stdout():
    if isinstance(sys.stdout, StringIO):
        del sys.stdout

    std = StringIO()
    sys.stdout = std

def assert_stdout(expected):
    string = sys.stdout.getvalue()
    assert_equals(string, expected)

def test_ansify():
    "couleur.ansify wraps ansi code for proper pythonic output"
    assert_equals(ansify(0), '\033[0m')

@with_setup(prepare_stdout)
def test_output_black_foreground():
    "Test output: black foreground"
    sh = Shell()
    sh.black("Hello Black!")
    assert_stdout('\033[30mHello Black!\033[0m')

@with_setup(prepare_stdout)
def test_output_black_foreground_on_white_background():
    "Test output: black foreground on white background"
    sh = Shell()
    sh.black_on_white("Hello Black!")
    assert_stdout('\033[47m\033[30mHello Black!\033[0m')

@with_setup(prepare_stdout)
def test_output_green_foreground():
    "Test output: green foreground"
    sh = Shell()
    sh.green("Hello World!")
    assert_stdout('\033[32mHello World!\033[0m')

@with_setup(prepare_stdout)
def test_mixed_output():
    "green_and_red_and_white is a valid call"
    sh = Shell()
    sh.green_and_red_and_white("Hello |World |for you!")
    assert_stdout(
        '\033[32mHello \033[0m\033[31mWorld \033[0m\033[37mfor you!\033[0m'
    )

@with_setup(prepare_stdout)
def test_mixed_output_with_escaped_separator():
    "green_and_red_on_yellow works is a valid call"
    sh = Shell()
    sh.green_and_red_on_yellow("Hello |World \|for you!")
    assert_stdout(
        '\033[32mHello \033[0m\033[43m\033[31mWorld |for you!\033[0m'
    )

@with_setup(prepare_stdout)
def test_mixed_output_with_backgrounds():
    "green_on_magenta_and_red_and_white_on_blue is a valid call"
    sh = Shell()
    sh.green_on_magenta_and_red_and_white_on_blue("Hello |World |for you!")
    assert_stdout('\033[45m\033[32mHello \033[0m\033[31mWorld \033[0m\033[44m\033[37mfor you!\033[0m'
    )

@with_setup(prepare_stdout)
def test_indent():
    "indentation"
    sh = Shell(indent=4, linebreak=True)
    sh.normal_on_blue("Hello")
    sh.indent()
    sh.normal_on_red("World")
    assert_stdout('\033[44m\033[39mHello\033[0m\n    \033[41m\033[39mWorld\033[0m\n')

@with_setup(prepare_stdout)
def test_dedent():
    "de-indentation"
    sh = Shell(indent=4, linebreak=True)
    sh.indent()
    sh.normal_on_blue("Hello")
    sh.dedent()
    sh.normal_on_red("World")
    assert_stdout('    \033[44m\033[39mHello\033[0m\n\033[41m\033[39mWorld\033[0m\n')

@with_setup(prepare_stdout)
def test_bold():
    "bold text"
    sh = Shell(bold=True)
    sh.normal_on_blue("Hello")
    sh.normal_on_red("World")
    assert_stdout('\033[1m\033[44m\033[39mHello\033[0m\033[1m\033[41m\033[39mWorld\033[0m')

@with_setup(prepare_stdout)
def test_bold_inline():
    "bold text with inline call"
    sh = Shell()
    sh.bold_normal_on_blue("Hello")
    sh.bold_normal_on_red("World")
    assert_stdout('\033[44m\033[1m\033[39mHello\033[0m\033[41m\033[1m\033[39mWorld\033[0m')

@with_setup(prepare_stdout)
def test_update_shell():
    "updating the shell, replacing the last output"
    sh = Shell(indent=6)
    sh.yellow("Yellow")
    sh.indent()
    sh.red("Red", True)
    assert_stdout('\033[33mYellow\033[0m\r\033[A      \033[31mRed\033[0m')

@with_setup(prepare_stdout)
def test_update_shell_mixed_with_linebreak():
    "updating the shell with mixed output and linebreak enabled"
    sh = Shell(linebreak=True)
    sh.yellow("Yellow")
    sh.yellow_and_normal_and_red("Yellow| and |Red", True)
    sh.green("Green")
    assert_stdout('\033[33mYellow\033[0m\n\r\033[A\033[33mYellow\033[0m\033[39m and \033[0m\033[31mRed\033[0m\n\033[32mGreen\033[0m\n')

@with_setup(prepare_stdout)
def test_update_shell_mixed_with_indentation():
    "updating the shell with mixed output and indentation"
    sh = Shell(linebreak=True)
    sh.yellow("Yellow")
    sh.indent()
    sh.yellow_and_normal_and_red("Yellow| and |Red", True)
    sh.dedent()
    sh.green("Green")
    assert_stdout('\033[33mYellow\033[0m\n\r\033[A  \033[33mYellow\033[0m\033[39m and \033[0m\033[31mRed\033[0m\n\033[32mGreen\033[0m\n')

@with_setup(prepare_stdout)
def test_update_shell_mixed_with_bold():
    "updating the shell with mixed output and bold enabled"
    sh = Shell(bold=True)
    sh.yellow("Yellow")
    sh.yellow_and_normal_and_red("Yellow| and |Red", True)
    sh.green("Green")
    assert_stdout('\033[1m\033[33mYellow\033[0m\r\033[A\033[1m\033[33mYellow\033[0m\033[39m and \033[0m\033[31mRed\033[0m\033[1m\033[32mGreen\033[0m')
